@@ -18,7 +18,7 @@ from .tempdir import create_temporary_dir
 from .files import FileOperations
 from . import results
 from .io import IoHandler, Channel
-from .errors import NoSuchCommandError, CommandInitializationError
+from .errors import NoSuchCommandError, CommandInitializationError, CouldNotChangeDirectoryError
 
 
 _ONE_MINUTE = 60
@@ -80,21 +80,22 @@ class ShShellType(object):
             commands.append("echo $$")
 
         if cwd is not None:
-            commands.append("cd {0}".format(escape_sh(cwd)))
-        
+            commands.append("cd {0} 2>&1 || {{ echo '\n'spur-cd: $?; exit 1; }}".format(escape_sh(cwd)))
+            commands.append("echo '\n'spur-cd: 0")
+
         update_env_commands = [
             "export {0}={1}".format(key, escape_sh(value))
             for key, value in _iteritems(update_env)
         ]
         commands += update_env_commands
-        commands.append(" || ".join(self._generate_which_commands(command_args[0])))
-        commands.append("echo $?")
+        which_commands = " || ".join(self._generate_which_commands(command_args[0]))
+        which_commands = "{ { " + which_commands + "; } && echo 0; } || { echo $?; exit 1; }"
+        commands.append(which_commands)
         
         command = " ".join(map(escape_sh, command_args))
         command = "exec {0}".format(command)
         if new_process_group:
             command = "setsid {0}".format(command)
-            
         commands.append(command)
         return "; ".join(commands)
     
@@ -119,14 +120,18 @@ class SshShell(object):
             hostname,
             username=None,
             password=None,
-            port=22,
+            port=None,
             private_key_file=None,
             connect_timeout=None,
             missing_host_key=None,
             shell_type=None,
             look_for_private_keys=True,
-            load_system_host_keys=True):
+            load_system_host_keys=True,
+            sock=None):
         
+        if port is None:
+            port = 22
+
         if shell_type is None:
             shell_type = ShellTypes.sh
         
@@ -140,7 +145,8 @@ class SshShell(object):
         self._look_for_private_keys = look_for_private_keys
         self._load_system_host_keys = load_system_host_keys
         self._closed = False
-        
+        self._sock = sock
+
         if missing_host_key is None:
             self._missing_host_key = MissingHostKey.raise_error
         else:
@@ -166,6 +172,7 @@ class SshShell(object):
         store_pid = kwargs.pop("store_pid", False)
         use_pty = kwargs.pop("use_pty", False)
         encoding = kwargs.pop("encoding", None)
+        cwd = kwargs.get('cwd')
         command_in_cwd = self._shell_type.generate_run_command(command, *args, store_pid=store_pid, **kwargs)
         try:
             channel = self._get_ssh_transport().open_session()
@@ -179,7 +186,19 @@ class SshShell(object):
         
         if store_pid:
             pid = _read_int_initialization_line(process_stdout)
-        
+
+        if cwd is not None:
+            cd_output = []
+            while True:
+                line = process_stdout.readline()
+                if line.startswith(b"spur-cd: "):
+                    if line.strip() == b"spur-cd: 0":
+                        break
+                    else:
+                        raise CouldNotChangeDirectoryError(cwd, b"".join(cd_output))
+                else:
+                    cd_output.append(line)
+
         if self._shell_type.supports_which:
             which_return_code = _read_int_initialization_line(process_stdout)
             
@@ -202,7 +221,7 @@ class SshShell(object):
     
     @contextlib.contextmanager
     def temporary_dir(self):
-        result = self.run(["mktemp", "--directory"])
+        result = self.run(["mktemp", "--directory"], encoding="ascii")
         temp_dir = result.output.strip()
         try:
             yield temp_dir
@@ -263,7 +282,8 @@ class SshShell(object):
                 password=self._password,
                 key_filename=self._private_key_file,
                 look_for_keys=self._look_for_private_keys,
-                timeout=self._connect_timeout
+                timeout=self._connect_timeout,
+                sock=self._sock
             )
             self._client = client
         return self._client
