@@ -1,9 +1,11 @@
 # coding=utf8
 
+import errno
 import io
 import time
 import signal
 import functools
+import posixpath
 
 from nose.tools import istest, nottest, assert_equal, assert_not_equal, assert_raises, assert_true
 
@@ -276,6 +278,58 @@ class ProcessTestSet(object):
         # Get the output twice since the pty echoes input
         assert_equal(b"hello\r\nhello\r\n", result.output)
 
+    @test
+    def using_non_existent_cwd_raises_could_not_change_directory_error(shell):
+        cwd = "/some/silly/path"
+        try:
+            shell.spawn(["echo", "1"], cwd=cwd)
+            # Expected exception
+            assert False
+        except spur.CouldNotChangeDirectoryError as error:
+            assert_equal("Could not change directory to: {0}".format(cwd), error.args[0].split("\n")[0])
+            assert_equal(cwd, error.directory)
+
+    @test
+    def attempting_to_change_directory_without_permissions_raises_cannot_change_directory_error(shell):
+        with shell.temporary_dir() as temp_dir:
+            dir_without_execute_permissions = posixpath.join(temp_dir, "a")
+            shell.run(["mkdir", dir_without_execute_permissions])
+            shell.run(["chmod", "-x", dir_without_execute_permissions])
+            try:
+                shell.spawn(["true"], cwd=dir_without_execute_permissions)
+                # Expected exception
+                assert False
+            except spur.CouldNotChangeDirectoryError as error:
+                assert_equal(dir_without_execute_permissions, error.directory)
+
+    @test
+    def using_non_existent_cwd_and_command_raises_could_not_change_directory_error(shell):
+        try:
+            shell.spawn(["bin/i-am-not-a-command"], cwd="/some/silly/path")
+            # Expected exception
+            assert False
+        except spur.CouldNotChangeDirectoryError as error:
+            assert_equal("/some/silly/path", error.directory)
+
+    @test
+    def using_non_existent_command_and_correct_cwd_raises_no_such_command_exception(shell):
+        try:
+            shell.spawn(["bin/i-am-not-a-command"], cwd="/bin")
+            # Expected exception
+            assert False
+        except spur.NoSuchCommandError as error:
+            assert_equal("bin/i-am-not-a-command", error.command)
+
+    @test
+    def can_find_command_in_cwd(shell):
+        # TODO: the behaviour in subprocess seems to be inconsistent between
+        # both Python versions and platforms (Windows vs Unix)
+        # See:
+        # * https://bugs.python.org/issue15533
+        # * https://bugs.python.org/issue20927
+        result = shell.run(["./ls"], cwd="/bin")
+        assert_equal(result.return_code, 0)
+
 
 # TODO: timeouts in wait_for_result
 
